searchNodes=[{"doc":"This is a library of assertions for Elixir's ExUnit. It emphasizes two things:Making tests easier to scan by capturing frequently-used assertions in functions that can be used in a pipeline.This library will appeal to people who prefer this: VM.ServiceGap.accept_form(params, @institution) |&gt; ok_content |&gt; assert_valid |&gt; assert_changes(id: 1, in_service_datestring: @iso_date_1, out_of_service_datestring: @iso_date_2, reason: &quot;reason&quot;)... to this: assert {:ok, changeset} = VM.ServiceGap.accept_form(params, @institution) assert changeset.valid? changes = changeset.changes assert changes.id == 1 assert changes.in_service_datestring == @iso_date_1 assert changes.out_of_service_datestring == @iso_date_2 assert changes.reason == &quot;reason&quot;The key point here is that all of the assert_* functions in this package return their first argument to be used with later chained functions.Error messages as helpful as those in the base ExUnit assertions:InstallationAdd flow_assertions to your list of dependencies in mix.exs:def deps do [ {:flow_assertions, &quot;~&gt; 0.6&quot;, only: :test}, ] endUseThe easiest way is use FlowAssertions, which imports the most important modules, which are:FlowAssertions.MapAFlowAssertions.MiscAFlowAssertions.EnumAFlowAssertions.StructA(in roughly that order). If you prefer to alias rather than import, note that all the assertion modules end in A. That way, there's no conflict between the module with map assertions (FlowAssertions.MapA and the Map module itself.Reading error outputExUnit has very nice reporting for assertions where a left-hand side is compared to a right-hand side, as in:assert x == yThe error output shows the values of both x and y, using color-coding to highlight differences.FlowAssertions uses that mechanism when appropriate. However, it does more complicated comparisons, so the words left and right aren't strictly accurate. So, suppose you're reading errors from code like this:calculation |&gt; assert_something(expected) |&gt; assert_something_else(expected)In the output, left will refer to some value extracted from calculation and right will refer to a value extracted from expected (most likely expected itself).Defining your own assertionsTBDRelated codeTBDChange logHere.","ref":"FlowAssertions.html","title":"FlowAssertions","type":"module"},{"doc":"Assertions used to test other assertions.These assertions do not work on the textual output of an assertion error. Instead, they inspect the internals of the ExUnit.AssertionError structure. Here is an example: assertion_fails( &quot;some text&quot; [left: &quot;something&quot;], fn -&gt; assert_no_value(data, :something_field) end)The first argument (&quot;some text&quot;) describes the expected :message field. The next line describes the value expected in the :left field. As far as I know, the ExUnit.AssertionError isn't guaranteed to stay stable, so beware. As of August 2020, these are the fields you can use:defexception left: @no_value, right: @no_value, message: @no_value, expr: @no_value, args: @no_value, doctest: @no_value, context: :expr","ref":"FlowAssertions.AssertionA.html","title":"FlowAssertions.AssertionA","type":"module"},{"doc":"Check if an assertion fails in the expected way.The assertion must be wrapped in a function: fn -&gt; assert_no_value(data, :something_field) end)A typical call looks like this: assertion_fails( &quot;some text&quot; [left: &quot;something&quot;], fn -&gt; ... end)The first argument is the expected :message field in the ExUnit.AssertionError structure.The second argument is a keyword list describing the other fields of the AssertionError structure.The :message value and the other field values will be compared using FlowAssertions.MiscA.good_enough?/2. So, for example, the check of the error message need not be exact: assertion_fails( ~r/some text.*final text/, ...If the assertion correctly fails, the return value is the ExUnit.AssertionError the assertion created.","ref":"FlowAssertions.AssertionA.html#assertion_fails/3","title":"FlowAssertions.AssertionA.assertion_fails/3","type":"function"},{"doc":"A variant of assertion_fails/3 that is useful in chaining.This assertion is tailored for a function that is passed through a pipeline and exposed to different arguments at each stage: msg = &quot;...&quot; (&amp;error_content/1) # &lt;= parentheses are important |&gt; assertion_fails_for(:error, msg) |&gt; assertion_fails_for({:ok, &quot;content&quot;}, msg)In the above case, FlowAssertions.MiscA.error_content/1 is exposed to two different values. Both should cause an ExUnit.AssertionError with a particular :message.A third argument could describe other ExUnit.AssertionError fields, as in assertion_fails/3.Note the parentheses around error_content/1. They are required by Elixir's precedence rules: &amp; binds more loosely than |&gt;. If you leave the parentheses off, the expression turns into a single function that's never executed - meaning that the test can never fail, no matter how broken the code is.","ref":"FlowAssertions.AssertionA.html#assertion_fails_for/4","title":"FlowAssertions.AssertionA.assertion_fails_for/4","type":"function"},{"doc":"Functions that create handy predicates for use with FlowAssertions.MiscA.assert_good_enough/2actual |&gt; assert_good_enough( in_any_order([1, 2, 3]))&quot;Checkers&quot; typically provide custom failure messages that are better than what a simple predicate would provide.This module is a work in progress, but what now works will continue to work and can be used safely. For examples of what checkers might come, see the Midje documentation. (Links are under the third bullet point of &quot;The Core&quot; on that page.)","ref":"FlowAssertions.Checkers.html","title":"FlowAssertions.Checkers","type":"module"},{"doc":"Checks whether a String or List contains another.[1, 2, 3] |&gt; assert_good_enough( has_slice([2, 3])) &quot;abcdefg&quot; |&gt; assert_good_enough( has_slice(&quot;abc&quot;))","ref":"FlowAssertions.Checkers.html#has_slice/1","title":"FlowAssertions.Checkers.has_slice/1","type":"function"},{"doc":"Check equality of Enumerables, ignoring order.actual |&gt; assert_good_enough( in_any_order([1, 2, 3]))In case of error, the actual and expected enumerables are sorted by by their Kernel.inspect/1 representation. In combination with ExUnit's color-coded differences, that makes it easier to see what went wrong.","ref":"FlowAssertions.Checkers.html#in_any_order/1","title":"FlowAssertions.Checkers.in_any_order/1","type":"function"},{"doc":"use this module to get all the assertion-definition helpers at once.Specifically:ExUnit.AssertionsExUnit.AssertionError (as an alias)FlowAssertions.Define.Defchain (the defchain macro)FlowAssertions.Define.BodyParts (functions useful in assertion bodies)","ref":"FlowAssertions.Define.html","title":"FlowAssertions.Define","type":"module"},{"doc":"Functions helpful in the construction of a new assertion.Mostly, they give you more control over what's shown in a failing test by letting you set ExUnit.AssertionError values like :left and :right.All such functions take a string first argument. That's shorthand for setting the :message field.","ref":"FlowAssertions.Define.BodyParts.html","title":"FlowAssertions.Define.BodyParts","type":"module"},{"doc":"Run a function, perhaps generating an assertion error. If so, use the keyword arguments to replace or update values in the error. Replacement:adjust_assertion_error(fn -&gt; MiscA.assert_good_enough(Map.get(kvs, key), expected) end, message: &quot;Field `\#{inspect key}` has the wrong value&quot;, expr: AssertionError.no_value)Setting the expr field to AssertionError.no_value has the handy effect of making the reporting machinery report the code of the assertion the user called, rather than the nested assertion that generated the error.Update:adjust_assertion_error(fn -&gt; MiscA.assert_good_enough(Map.get(kvs, key), expected) end, expr: fn expr -&gt; [expr, &quot;...&quot;] end) # indicate something missing.See also adjust_assertion_message/2","ref":"FlowAssertions.Define.BodyParts.html#adjust_assertion_error/2","title":"FlowAssertions.Define.BodyParts.adjust_assertion_error/2","type":"function"},{"doc":"Run a function, perhaps generating an assertion error. If so, call the second function, passing the current assertion message as its argument. The result is installed as the new assertion message. adjust_assertion_message( fn -&gt; flunk &quot;message&quot; end, fn message -&gt; &quot;\#{message} and \#{message}&quot; end)See also adjust_assertion_error/2&quot;","ref":"FlowAssertions.Define.BodyParts.html#adjust_assertion_message/2","title":"FlowAssertions.Define.BodyParts.adjust_assertion_message/2","type":"function"},{"doc":"Like ExUnit.Assertions.assert/2 but the third argument is used to set AssertionError keys. elaborate_assert( left =~ right, &quot;Regular expression didn&#39;t match&quot;, left: left, right: right) Warning: as far as I know, the structure of ExUnit.AssertionError is not guaranteed to be stable.See also elaborate_assert_equal/4.","ref":"FlowAssertions.Define.BodyParts.html#elaborate_assert/3","title":"FlowAssertions.Define.BodyParts.elaborate_assert/3","type":"function"},{"doc":"This replicates the diagnostic output from assert a == b, except for the code snippet that's reported.The user will see a failing test containing:Assertion with == failed code: assert_same_map(new, old, ignoring: [:stable]) left: ... right: ...... instead of the assertion that actually failed, something like this:Assertion with == failed code: assert Map.drop(new, fields_to_ignore) == Map.drop(old, fields_to_ignore) left: ... right: ...","ref":"FlowAssertions.Define.BodyParts.html#elaborate_assert_equal/2","title":"FlowAssertions.Define.BodyParts.elaborate_assert_equal/2","type":"function"},{"doc":"Like ExUnit.Assertions.flunk/1 but the second argument is used to set AssertionError keys. elaborate_flunk(&quot;the value is wrong&quot;, left: value_to_check)Warning: as far as I know, the structure of ExUnit.AssertionError is not guaranteed to be stable.See also elaborate_assert/3.","ref":"FlowAssertions.Define.BodyParts.html#elaborate_flunk/2","title":"FlowAssertions.Define.BodyParts.elaborate_flunk/2","type":"function"},{"doc":"elaborate_assert/3, except the value is expected to be falsy.","ref":"FlowAssertions.Define.BodyParts.html#elaborate_refute/3","title":"FlowAssertions.Define.BodyParts.elaborate_refute/3","type":"function"},{"doc":"Flunk test if it checks structure fields that don't exist.It doesn't make sense to write an assertion that checks a field that a structure can't contain. If a user tries, this function will object with a message like: Test error: there is no key `:b` in a `MyApp.Struct`Notes:It's safe to call on non-struct values.It returns its first argument.","ref":"FlowAssertions.Define.BodyParts.html#struct_must_have_key!/2","title":"FlowAssertions.Define.BodyParts.struct_must_have_key!/2","type":"function"},{"doc":"Same as struct_must_have_key!/2 but checks multiple keys.","ref":"FlowAssertions.Define.BodyParts.html#struct_must_have_keys!/2","title":"FlowAssertions.Define.BodyParts.struct_must_have_keys!/2","type":"function"},{"doc":"Easier creation of assertions that can be chained together.That looks like this:get(conn, ...) |&gt; assert_will_post_to(:set_fresh_password) |&gt; assert_user_sees(required_1) |&gt; assert_user_sees(required_2)","ref":"FlowAssertions.Define.Defchain.html","title":"FlowAssertions.Define.Defchain","type":"module"},{"doc":"Assertions are created by using defchain instead of def.defchain assert_user_sees(conn, claim), do: ... defchain assert_purpose(conn, purpose), do: ...","ref":"FlowAssertions.Define.Defchain.html#defchain/2","title":"FlowAssertions.Define.Defchain.defchain/2","type":"macro"},{"doc":"Code to support writing checkers like the ones in FlowAssertions.Checkers.Note that this is a work in progress, so be prepared for change.","ref":"FlowAssertions.Define.Defchecker.html","title":"FlowAssertions.Define.Defchecker","type":"module"},{"doc":"","ref":"FlowAssertions.Define.Defchecker.html#fail_helpfully/1","title":"FlowAssertions.Define.Defchecker.fail_helpfully/1","type":"function"},{"doc":"","ref":"FlowAssertions.Define.Defchecker.html#fail_helpfully/2","title":"FlowAssertions.Define.Defchecker.fail_helpfully/2","type":"function"},{"doc":"","ref":"FlowAssertions.Define.Defchecker.html#fail_helpfully/3","title":"FlowAssertions.Define.Defchecker.fail_helpfully/3","type":"function"},{"doc":"Generate &quot;runners&quot; used in tabular tests of assertions and assertion-like functions. A typical example: a = assertion_runners_for(&amp;assert_equal/2) [&quot;value&quot;, &quot;value&quot;] |&gt; a.pass.() [&quot;value&quot;, &quot; &quot;] |&gt; a.fail.(&quot;Assertion with === failed&quot;) |&gt; a.plus.(left: &quot;value&quot;, right: &quot; &quot;)assert_equal takes two arguments, and those are passed to the runner functions (pass and fail) in a list. If there's only a single argument, the enclosing list can be omitted: a = assertion_runners_for(&amp;assert_ok/1) :ok |&gt; a.pass.() :error |&gt; a.fail.(Messages.not_ok)The functions created are, unless noted in the individual descriptions below, these:pass: A runner that applies a function to arguments and checks for success. What, precisely, counts as success depends on the function under test. For a flow assertion, success means returning its first argument.In the examples above, pass took only one argument (the value on the left side of the |&gt;). That's because the expected output of assert_ok is trivial. In other cases, you must give the expected output to pass. That looks like this:test &quot;ok_content&quot; do a = content_runners_for(&amp;ok_content/1) {:ok, &quot;content&quot;} |&gt; a.pass.(&quot;content&quot;) ...fail: takes an argument that is matched against an AssertionError. A string, which must match the error message exactly. A regular expression, like ~r/different lengths/, which can match part of the message.A keyword list, which is checked with FlowAssertions.MapA.assert_fields/2. A typical use might be: [ {1, 2}, 3 ] |&gt; a.fail.(left: {1, 2}, right: 3)Note that the left and right values are the true values from ExUnit.AssertionError, not string versions as from inspect. If the error message is also to be tested, it should be given in keyword form: a.fail.(left: &quot;a&quot;, right: &quot;bb&quot;, message: ~r/different lengths/)Warning: field names like :left, :right, and :message are not part of the public API of ExUnit.AssertionError, so those names could change in the future (though I doubt it). See the AssertionError source for some other fields.plus: Provides an arguably nicer way to check both the :message and other AssertionError fields. It's appended to fail like this: [datetime] |&gt; a.fail.(Messages.wrong_struct_name(NaiveDateTime, Date)) |&gt; a.plus.(left: datetime)inspect_. This is a function to help with test-writing workflow. It mainly exists because it's hard for me to get the display of error information right without looking at it. That is, a check like this: [ [1, 2, 3], [7, 1, 3] ] |&gt; a.fail.(~r/different elements/) |&gt; a.plus.(...)is likely to come from looking a message like the one at the end of this bullet list, then tweaking the message and maybe the code:, left:, or right: output. I only &quot;solidify&quot; the final design in a test after everything looks right. Normally, producing such error output would mean writing throwaway code, but inspect_ avoids that. Once I pick test inputs (the values before the |&gt;), I just write this: [ [1, 2, 3], [7, 1, 3] ] |&gt; a.inspect_.(~r/a first message version/) ^^^^^^^^The trailing _ indicates that inspect_ is replacing a function that takes one argument. Use inspect to replace a function with zero arguments, like pass. Beware the typoThe most common mistake I make with this library is this kind of typo: [&quot;&quot;, &quot;a&quot; ] |&gt; a.fail(~s/Checker `has_slice(&quot;a&quot;)` failed/)There should be a period after fail. The result (as of Elixir 1.11) is ** (ArgumentError) you attempted to apply a function on %{arity: 1, fail: #Function&lt;5.65840318/2 in FlowAssertions.Define.Tabular.make_ assertion_fail/1&gt;, inspect: #Function&lt;0.65840318/1 in FlowAssertion ... Assertions.Define.Tabular.start/2&gt;}. Modules (the first argument of apply) must always be an atom... which is perhaps not as clear as it should be. But now you're forewarned.","ref":"FlowAssertions.Define.Tabular.html","title":"FlowAssertions.Define.Tabular","type":"module"},{"doc":"Create runners for flow-style assertions.a = assertion_runners_for(&amp;assert_empty/1)asserter should be a function that either raises an AssertionError or returns its first argument. pass and fail are as described above.","ref":"FlowAssertions.Define.Tabular.html#assertion_runners_for/1","title":"FlowAssertions.Define.Tabular.assertion_runners_for/1","type":"function"},{"doc":"Create runners for functions like those in FlowAssertions.Checkers. a = checker_runners_for(&amp;in_any_order/1)] # actual checked against [ [1, 2, 3], [1, 2, 3] ] |&gt; a.pass.() [ [1, 2, 3], [7, 1, 3] ] |&gt; a.fail.(~r/different elements/)checker should be a function that can return a FlowAssertions.Define.Defchecker.Failure value. As for what that means... Well, as of late 2020, checker creation is not documented.","ref":"FlowAssertions.Define.Tabular.html#checker_runners_for/1","title":"FlowAssertions.Define.Tabular.checker_runners_for/1","type":"function"},{"doc":"Create runners for functions that either return part of a compound value or raise an AssertionError. Consider FlowAssertions.MiscA.ok_content: a = content_runners_for(&amp;ok_content/1) {:ok, &quot;content&quot;} |&gt; a.pass.(&quot;content&quot;) :ok |&gt; a.fail.(Messages.not_ok_tuple) {:error, &quot;content&quot;} |&gt; a.fail.(Messages.not_ok_tuple)Note that pass takes a single value.","ref":"FlowAssertions.Define.Tabular.html#content_runners_for/1","title":"FlowAssertions.Define.Tabular.content_runners_for/1","type":"function"},{"doc":"Adjust the results of a fail function to assert a value for left:. Suppose you create tabular functions for FlowAssertions.MiscA.ok_content like this: a = content_runners_for(&amp;ok_content/1) |&gt; left_is_actual ^^^^^^^^^^^^^^^^^Then a table entry like this: :ok |&gt; a.fail.(Messages.not_ok_tuple)... will check that the :left value of the AssertionError is :ok. (That is, it is the argument given to ok_content). There is no need to add a |&gt; plus.(left: :ok). If the function under test takes more than one argument, the left: value must be the first element of the list on the left-hand side of the |&gt;. That is, in the following, the left: value is checked to be [1]. a = assertion_runners_for(&amp;assert_equal/2) |&gt; left_is_actual [[1], 1] |&gt; x.fail.(&quot;Assertion with === failed&quot;)","ref":"FlowAssertions.Define.Tabular.html#left_is_actual/1","title":"FlowAssertions.Define.Tabular.left_is_actual/1","type":"function"},{"doc":"Create runners for regular Elixir assertions.asserter should be a function that raises an AssertionError or returns an unspecified value. pass, then, always succeeds when there's no error.","ref":"FlowAssertions.Define.Tabular.html#nonflow_assertion_runners_for/1","title":"FlowAssertions.Define.Tabular.nonflow_assertion_runners_for/1","type":"function"},{"doc":"Assertions that apply to Enums.","ref":"FlowAssertions.EnumA.html","title":"FlowAssertions.EnumA","type":"module"},{"doc":"Assert that an Enum has no elements.&quot;[] |&gt; assert_empty # true %{} |&gt; assert_empty # true","ref":"FlowAssertions.EnumA.html#assert_empty/1","title":"FlowAssertions.EnumA.assert_empty/1","type":"function"},{"doc":"If the value doesn't implement Enumerable produces an assertion exception.The output is more friendly than a Protocol.UndefinedError. So, for example, the other assertions in this module start with this assertion.defchain assert_empty(value_to_check) do assert_enumerable(value_to_check) ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ elaborate_assert(Enum.empty?(value_to_check), Messages.expected_no_element, left: value_to_check) end","ref":"FlowAssertions.EnumA.html#assert_enumerable/1","title":"FlowAssertions.EnumA.assert_enumerable/1","type":"function"},{"doc":"Assert that an Enum has only a single element.[1] |&gt; assert_singleton # passes [ ] |&gt; assert_singleton # fails %{a: 1} |&gt; assert_singleton # passes %{a: 1, b: 2} |&gt; assert_singleton # fails","ref":"FlowAssertions.EnumA.html#assert_singleton/1","title":"FlowAssertions.EnumA.assert_singleton/1","type":"function"},{"doc":"Returns the content element of what must be a single-element Enum.''' [1] |&gt; singleton_content # 1 [ ] |&gt; singleton_content # fails %{a: 1} |&gt; singleton_content # the tuple {:a, 1} 5 |&gt; singleton_content # faila","ref":"FlowAssertions.EnumA.html#singleton_content/1","title":"FlowAssertions.EnumA.singleton_content/1","type":"function"},{"doc":"Combines singleton_content/1 and FlowAssertions.StructA.assert_struct_named/2. |&gt; VM.Animal.lift(Schema.Animal) |&gt; singleton_content(VM.Animal) |&gt; assert_fields(...) In addition to checking that the value is a singleton Enumerable, it checks that the content is a value of the named struct before returning it.","ref":"FlowAssertions.EnumA.html#singleton_content/2","title":"FlowAssertions.EnumA.singleton_content/2","type":"function"},{"doc":"Assertions that apply to Maps and structures and sometimes to keyword lists.assert_fields/2 and assert_same_map/3 are the most important.","ref":"FlowAssertions.MapA.html","title":"FlowAssertions.MapA","type":"module"},{"doc":"Same as assert_fields/2 but more pleasingly grammatical when testing only one field:assert_field(some_map, key: &quot;value&quot;)When checking existence, you don't have to use a list:assert_field(some_map, :key)","ref":"FlowAssertions.MapA.html#assert_field/2","title":"FlowAssertions.MapA.assert_field/2","type":"function"},{"doc":"Assert that the value of the map at the key matches a binding form. assert_field_shape(map, :field, %User{}) assert_field_shape(map, :field, [_ | _])See FlowAssertions.MiscA.assert_shape/2 for more.","ref":"FlowAssertions.MapA.html#assert_field_shape/3","title":"FlowAssertions.MapA.assert_field_shape/3","type":"macro"},{"doc":"Test the existence and value of multiple fields with a single assertion:assert_fields(some_map, key1: 12, key2: &quot;hello&quot;)You can test just for existence:assert_fields(some_map, [:key1, :key2]The keyword list need not contain all of the fields in some_map.Values in the keyword list are compared as with FlowAssertions.MiscA.assert_good_enough/2. For example, regular expressions can be used to check strings:assert_fields(some_map, name: ~r/_cohort/)assert_fields can also take a map as its second argument. That's useful when the map to be tested has non-keyword arguments:assert_fields(string_map, %{&quot;a&quot; =&gt; 3})","ref":"FlowAssertions.MapA.html#assert_fields/2","title":"FlowAssertions.MapA.assert_fields/2","type":"function"},{"doc":"An equality comparison of two maps that gives control over which fields should not be compared or should be compared differently. It is typically used after some old map has been transformed to make a new one. To exclude some fields from the comparison: assert_same_map(new, old, ignoring: [:lock_version, :updated_at]) To compare only some of the keys: assert_same_map(new, old, comparing: [:name, :people]) To assert different values for particular fields: assert_same_map(new, old, except: [lock_version: old.lock_version + 1, people: &amp;Enum.empty/1]) Note that the except comparison uses FlowAssertions.MiscA.assert_good_enough/2. Note that if the first value is a struct, the second must have the same type: Assertion with == failed left: %S{b: 3} right: %R{b: 3}","ref":"FlowAssertions.MapA.html#assert_same_map/3","title":"FlowAssertions.MapA.assert_same_map/3","type":"function"},{"doc":"Same as refute_fields/2, but for a single field.%{a: 1} |&gt; refute_field(:a) # fails","ref":"FlowAssertions.MapA.html#refute_field/2","title":"FlowAssertions.MapA.refute_field/2","type":"function"},{"doc":"Fail if any of the fields in the field_list are present.%{a: 1} |&gt; refute_fields([:a, :b]) # fails","ref":"FlowAssertions.MapA.html#refute_fields/2","title":"FlowAssertions.MapA.refute_fields/2","type":"function"},{"doc":"Take a map and a field. Return the single element in the field's value.with_singleton_content(%{a: [1]}, :a) # returns `1`This is typically used with fields that take list values. Often, you only want to test the empty list and a singleton list. (When testing functions that produce their values with Enum.map/2 or for, creating a second list element gains you nothing.) Using with_singleton_content, it's convenient to apply assertions to the single element:view_model |&gt; assert_assoc_loaded(:service_gaps) |&gt; with_singleton_content(:service_gaps) |&gt; assert_shape(%VM.ServiceGap{}) |&gt; Ex.Datespan.assert_datestrings(:first)If field does not exist or isn't an Enum, with_singleton_content will fail in the same way FlowAssertions.EnumA.singleton_content/1 does.","ref":"FlowAssertions.MapA.html#with_singleton_content/2","title":"FlowAssertions.MapA.with_singleton_content/2","type":"function"},{"doc":"Miscellaneous, including assertions for common idioms like {:ok, &lt;content&gt;}","ref":"FlowAssertions.MiscA.html","title":"FlowAssertions.MiscA","type":"module"},{"doc":"The chaining version of assert x === y.This is useful for cases where you're adding onto a pipeline of assertions or content-extraction checks (like ok_content/1).value_to_check |&gt; ok_content |&gt; assert_equal(3)Note that the comparison is done with ===, so 1 is not equal to 1.0.","ref":"FlowAssertions.MiscA.html#assert_equal/2","title":"FlowAssertions.MiscA.assert_equal/2","type":"function"},{"doc":"Synonym for assert_equal.","ref":"FlowAssertions.MiscA.html#assert_equals/2","title":"FlowAssertions.MiscA.assert_equals/2","type":"function"},{"doc":"Check if value is an :error or an {:error, &lt;content&gt;} tuple.value_to_check |&gt; assert_error(:error) value_to_check |&gt; assert_error({:error, &quot;any value is accepted&quot;})See also error_content/1, which takes an :error tuple and returns the second element.","ref":"FlowAssertions.MiscA.html#assert_error/1","title":"FlowAssertions.MiscA.assert_error/1","type":"function"},{"doc":"A variant of assert_error/2 for three-element :error tuples.value_to_check |&gt; assert_error(:error, :constraint)Sometimes it's useful for an :error tuple to identify different kinds of errors. For example, Phoenix form processing errors might be due to either :validation or :constraint errors and reported in a tuple like {:error, :constraint, &lt;message&gt;}This function checks that the second element is as required. The third element is ignored.See also error2_content/2, which takes such a tuple and returns the third element.","ref":"FlowAssertions.MiscA.html#assert_error2/2","title":"FlowAssertions.MiscA.assert_error2/2","type":"function"},{"doc":"Like assert x == y, but with special handling for predicates and regular expressions.By default assert_good_enough uses == to test the left side against the right. However:If the right side is a regular expression and the left side is not, the two are compared with =~ rather than ==.If the right side is a function and the left side is not, the function is applied to the value. Any &quot;falsy&quot; value is a failure.assert_good_enough?(1, &amp;odd/1) assert_good_enough?(&quot;string&quot;, ~r/s.r..g/)See also good_enough?/2","ref":"FlowAssertions.MiscA.html#assert_good_enough/2","title":"FlowAssertions.MiscA.assert_good_enough/2","type":"function"},{"doc":"Check if a value is an :ok or an {:ok, &lt;content&gt;} tuple.value_to_check |&gt; assert_ok(:ok) value_to_check |&gt; assert_ok({:ok, &quot;any value is accepted&quot;})See also ok_content/1, which takes an :ok tuple and returns the second element.","ref":"FlowAssertions.MiscA.html#assert_ok/1","title":"FlowAssertions.MiscA.assert_ok/1","type":"function"},{"doc":"Assert that the value matches a binding form. value_to_check |&gt; assert_shape(%User{}) value_to_check |&gt; assert_shape(thing, [_ | _])Note that this is a macro that uses the match operator, so all of Elixir's pattern matching is available. For example, you can use a map to partially match a structure:make_user(&quot;fred&quot;) |&gt; assert_shape(%{name: &quot;fred&quot;})Or you can pin a value: make_user(&quot;fred&quot;) |&gt; assert_shape(%{name: ^chosen_name})See also FlowAssertions.StructA.assert_struct_named/2.","ref":"FlowAssertions.MiscA.html#assert_shape/2","title":"FlowAssertions.MiscA.assert_shape/2","type":"macro"},{"doc":"Fail unless the value given is a three-element tuple with the first element :error and the second a required subcategory of error. Returns the third element. error_content is used to let the rest of an assertion chain operate on the content:|&gt; ReservationApi.create(ready, @institution) |&gt; error2_content(:constraint) |&gt; assert_equals(&quot;some error message&quot;)See also assert_error2/2.","ref":"FlowAssertions.MiscA.html#error2_content/2","title":"FlowAssertions.MiscA.error2_content/2","type":"function"},{"doc":"Fails with an AssertionError unless the argument is of the form {:error, content}. Returns the content value.error_content is used to let the rest of an assertion chain operate on the content:|&gt; ReservationApi.create(ready, @institution) |&gt; error_content |&gt; assert_equals(&quot;some error message&quot;)See also assert_error/1.","ref":"FlowAssertions.MiscA.html#error_content/1","title":"FlowAssertions.MiscA.error_content/1","type":"function"},{"doc":"Think of it as a form of equality with special handling for functions and regular expressions.good_enough?(1, &amp;odd/1) # true good_enough?(&quot;string&quot;, ~r/s.r..g/) # trueIf the second argument is a regular expression and the first is a string, the two are compared with =~. If both arguments are regular expressions, their source fields are compared.If the second argument is a function and the first is not, the function is applied to the first argument. good_enough? returns true iff the result is truthy. Otherwise, the two are compared with ==. See also assert_good_enough/2","ref":"FlowAssertions.MiscA.html#good_enough?/2","title":"FlowAssertions.MiscA.good_enough?/2","type":"function"},{"doc":"Fails with an AssertionError unless the argument is of the form {:ok, content}. Returns the content value.ok_content is used to let the rest of an assertion chain operate on the content:|&gt; ReservationApi.create(ready, @institution) |&gt; ok_content |&gt; assert_field(count: 5)See also assert_ok/1.","ref":"FlowAssertions.MiscA.html#ok_content/1","title":"FlowAssertions.MiscA.ok_content/1","type":"function"},{"doc":"Combines ok_content/1 and FlowAssertions.StructA.assert_struct_named/2. |&gt; VM.accept_form(params) |&gt; ok_content(Changeset) |&gt; assert_no_changesIn addition to checking that the value is an {:ok, content} tuple, it checks that the content is a value of the named struct before returning it.","ref":"FlowAssertions.MiscA.html#ok_content/2","title":"FlowAssertions.MiscA.ok_content/2","type":"function"},{"doc":"Extract the :id field from an {:ok, %{id: id, ...}} value.Shorthand for ok_content(x).id.","ref":"FlowAssertions.MiscA.html#ok_id/1","title":"FlowAssertions.MiscA.ok_id/1","type":"function"},{"doc":"These assertions assume a convention of initializing keys in a map to an &quot;I have no value&quot; value, with the expectation that they will later be given real values.Such a convention is useful in multi-step construction of, for example, ExUnit assertion errors. They are structures initialized to :ex_unit_no_meaningful_value. The values are then set by an assertion error. Moreover, they can be reset by code that rescues an error. Several functions in this package make use of that. See FlowAssertions.Define.BodyParts.adjust_assertion_error/2 for an example.Use this module with use, providing the no-value value:use FlowAssertions.NoValueA, no_value: :_nothing ... result |&gt; assert_no_value([:key1, key2])If you use the same no-value value widely, consider using this module once and importing that:defmodule My.NoValueA do use FlowAssertions.NoValueA, no_value: :ex_unit_no_meaningful_value end defmodule MyTestModule import My.NoValueA ... result |&gt; assert_no_value([:key1, key2])If you don't use use, you can provide the no-value value on each call:import FlowAssertions.NoValueA ... result |&gt; assert_no_value([:key1, :key2], :ex_unit_no_meaningful_value)The default no-value value is nil.","ref":"FlowAssertions.NoValueA.html","title":"FlowAssertions.NoValueA","type":"module"},{"doc":"Assert that one or more keys in a map have no value.Note that the second argument can be either a singleton key or a list.","ref":"FlowAssertions.NoValueA.html#assert_no_value/3","title":"FlowAssertions.NoValueA.assert_no_value/3","type":"function"},{"doc":"Assert that one or more keys in a map have been assigned values.Note that the second argument can be either a singleton key or a list.The optional third argument gives the &quot;value that is no value&quot;. It's used to signify that the structure has never had its initial value &quot;changed&quot;.","ref":"FlowAssertions.NoValueA.html#assert_values_assigned/3","title":"FlowAssertions.NoValueA.assert_values_assigned/3","type":"function"},{"doc":"","ref":"FlowAssertions.NoValueA.html#refute_no_value/3","title":"FlowAssertions.NoValueA.refute_no_value/3","type":"function"},{"doc":"Assertions that apply only to structs, not maps.","ref":"FlowAssertions.StructA.html","title":"FlowAssertions.StructA","type":"module"},{"doc":"Assert that the value is a particular module's struct.make_user(&quot;fred&quot;) |&gt; assert_struct_named(User)For structs, this has the same purpose as FlowAssertions.MiscA.assert_shape/2. However, because it's not a macro, the second argument can be a variable. That makes it useful for building up larger assertion functions.","ref":"FlowAssertions.StructA.html#assert_struct_named/2","title":"FlowAssertions.StructA.assert_struct_named/2","type":"function"},{"doc":"Changelog","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"AdditionsFlowassertions.Define.Tabular functions now have a stable enough API to use safely.FlowAssertions.Define.Tabular.nonflow_assertion_runners_for/1FlowAssertions.Checkers.has_slice/1TweaksFlowAssertions.Define.BodyParts.adjust_assertion_error/2 now combines both put and update! behavior for replacements.","ref":"changelog.html#0-6-0","title":"Changelog - 0.6.0","type":"extras"},{"doc":"AdditionsFlowAssertions.Define.BodyParts.adjust_assertion_message/2FlowAssertions.MapA.refute_fields/2TweaksFlowAssertions.Define.BodyParts.struct_must_have_key!/2 has a better error message.","ref":"changelog.html#0-5-0","title":"Changelog - 0.5.0","type":"extras"},{"doc":"FeaturesFlowAssertions.CheckersCheckers work with FlowAssertions.MiscA.assert_good_enough/2 in a way that allows better error messages.FlowAssertions.MiscA.ok_content/2 Like FlowAssertions.MiscA.ok_content/1, but takes a second, name argument. The content must be a struct with that name (a module).FlowAssertions.EnumA.singleton_content/2 Like FlowAssertions.EnumA.singleton_content/1, but takes a second, name argument. The content must be a struct with that name (a module).FlowAssertions.Define.BodyParts.elaborate_refute/3A variant of ExUnit.Assertions.refute/1 that allows creation of specialized error messages.DeprecationsDeprecate FlowAssertions.NoValueA.refute_no_value/3 in favor of FlowAssertions.NoValueA.assert_values_assigned/3","ref":"changelog.html#0-4-0","title":"Changelog - 0.4.0","type":"extras"}]